/**
 * Many of these utility types and functions were taken directly from solid-headless
 * https://github.com/lxsmnsyc/solid-headless
 *
 * Some have since been changed but it's important to
 * attribute where its appropriate.
 */
import { JSX } from "solid-js";
export type OmitAndMerge<A, B> = A & Omit<B, keyof A>;
export type Accessor<T = unknown> = () => T;
export type Maybe<T> = NonNullable<T> | undefined;
export type ValidElements = keyof JSX.IntrinsicElements;
export type ValidComponent<P> = (props: P) => JSX.Element;
export type ValidConstructor = ValidElements | ValidComponent<any> | (string & {});
export type DynamicProps<T extends ValidConstructor> = T extends ValidElements ? JSX.IntrinsicElements[T] : T extends ValidComponent<infer U> ? U : Record<string, unknown>;
type UnboxIntrinsicElements<T> = T extends JSX.HTMLAttributes<infer U> ? U : never;
type RefCallback<T> = (el: T) => void;
type RefField<T> = T | RefCallback<T>;
type UnboxComponentProp<U> = U extends {
    ref: infer X;
} ? X : never;
export type DynamicNode<T extends ValidConstructor> = T extends ValidElements ? UnboxIntrinsicElements<JSX.IntrinsicElements[T]> : T extends ValidComponent<infer U> ? UnboxComponentProp<U> : never;
export interface WithRef<T extends ValidConstructor> {
    ref?: RefField<DynamicNode<T>>;
}
export interface DynamicComponentWithRef<T extends ValidConstructor> extends WithRef<T> {
    as?: T;
}
export type HeadlessPropsWithRef<T extends ValidConstructor, V = {}> = OmitAndMerge<V & DynamicComponentWithRef<T>, DynamicProps<T>>;
export declare function convertToMaybe<T>(value: T): Maybe<T>;
export declare function omitProps<T extends Record<string, any>, K extends keyof T>(value: T, keys: K[]): Omit<T, K>;
export default function createDynamic<T extends ValidConstructor>(source: () => T, props: DynamicProps<T>): JSX.Element;
export declare function createPropsFromAccessors<T extends Record<string, Accessor>>(props: T): {
    [P in keyof T]: ReturnType<T[P]>;
};
export {};
